@@ -9,36 +9,36 @@
 # testing and support for generic AT commads, so use it at your own risk,
 # and without ANY warranty! Have fun.
 #
-# $Id: Modem.pm,v 1.33 2004/01/23 00:28:25 cosimo Exp $
+# $Id: Modem.pm,v 1.35 2004/02/22 16:52:47 cosimo Exp $
 
 package Device::Modem;
-$VERSION = sprintf '%d.%02d', q$Revision: 1.33 $ =~ /(\d)\.(\d+)/;
+$VERSION = sprintf '%d.%02d', q$Revision: 1.35 $ =~ /(\d)\.(\d+)/;
 
 BEGIN {
 
-	if( $^O =~ /Win/io ) {
+    if( $^O =~ /Win/io ) {
 
-		require Win32::SerialPort;
-		import  Win32::SerialPort;
+        require Win32::SerialPort;
+        import  Win32::SerialPort;
 
-		# Import line status constants from Win32::SerialPort module
-		*Device::Modem::MS_CTS_ON  = *Win32::SerialPort::MS_CTS_ON;
-		*Device::Modem::MS_DSR_ON  = *Win32::SerialPort::MS_DSR_ON;
-		*Device::Modem::MS_RING_ON = *Win32::SerialPort::MS_RING_ON;
-		*Device::Modem::MS_RLSD_ON = *Win32::SerialPort::MS_RLSD_ON;
+        # Import line status constants from Win32::SerialPort module
+        *Device::Modem::MS_CTS_ON  = *Win32::SerialPort::MS_CTS_ON;
+        *Device::Modem::MS_DSR_ON  = *Win32::SerialPort::MS_DSR_ON;
+        *Device::Modem::MS_RING_ON = *Win32::SerialPort::MS_RING_ON;
+        *Device::Modem::MS_RLSD_ON = *Win32::SerialPort::MS_RLSD_ON;
 
-	} else {
+    } else {
 
-		require Device::SerialPort;
-		import  Device::SerialPort;
+        require Device::SerialPort;
+        import  Device::SerialPort;
 
-		# Import line status constants from Device::SerialPort module
-		*Device::Modem::MS_CTS_ON = *Device::SerialPort::MS_CTS_ON;
-		*Device::Modem::MS_DSR_ON = *Device::SerialPort::MS_DSR_ON;
-		*Device::Modem::MS_RING_ON = *Device::SerialPort::MS_RING_ON;
-		*Device::Modem::MS_RLSD_ON = *Device::SerialPort::MS_RLSD_ON;
+        # Import line status constants from Device::SerialPort module
+        *Device::Modem::MS_CTS_ON = *Device::SerialPort::MS_CTS_ON;
+        *Device::Modem::MS_DSR_ON = *Device::SerialPort::MS_DSR_ON;
+        *Device::Modem::MS_RING_ON = *Device::SerialPort::MS_RING_ON;
+        *Device::Modem::MS_RLSD_ON = *Device::SerialPort::MS_RLSD_ON;
 
-	}
+    }
 }
 
 use strict;
@@ -57,9 +57,9 @@ $Device::Modem::STOPBITS = 1;
 $Device::Modem::PARITY   = 'none';
 $Device::Modem::TIMEOUT  = 500;     # milliseconds;
 
-
 # Setup text and numerical response codes
 @Device::Modem::RESPONSE = ( 'OK', undef, 'RING', 'NO CARRIER', 'ERROR', undef, 'NO DIALTONE', 'BUSY' );
+
 #%Device::Modem::RESPONSE = (
 #	'OK'   => 'Command executed without errors',
 #	'RING' => 'Detected phone ring',
@@ -71,55 +71,56 @@ $Device::Modem::TIMEOUT  = 500;     # milliseconds;
 
 # object constructor (prepare only object)
 sub new {
-	my($proto,%aOpt) = @_;                  # Get reference to object
-	                                        # Options of object
-	my $class = ref($proto) || $proto;      # Get reference to class
+    my($proto,%aOpt) = @_;                  # Get reference to object
+    # Options of object
+    my $class = ref($proto) || $proto;      # Get reference to class
 
-	$aOpt{'ostype'} = $^O;                  # Store OSTYPE in object
-	$aOpt{'ostype'} =~ /Win/io and $aOpt{'ostype'} = 'windoze';
+    $aOpt{'ostype'} = $^O;                  # Store OSTYPE in object
+    $aOpt{'ostype'} =~ /Win/io and $aOpt{'ostype'} = 'windoze';
 
-	# Initialize flags array
-	$aOpt{'flags'} = {};
+    # Initialize flags array
+    $aOpt{'flags'} = {};
 
-	$aOpt{'port'} ||= $Device::Modem::DEFAULT_PORT;
+    $aOpt{'port'} ||= $Device::Modem::DEFAULT_PORT;
 
-	# Instance log object
-	$aOpt{'log'} ||= 'file';
+    # Instance log object
+    $aOpt{'log'} ||= 'file';
 
-	# Force logging to file if this is windoze and user requested syslog mechanism
-	$aOpt{'log'} = 'file' if( $aOpt{'ostype'} eq 'windoze' && $aOpt{'log'} =~ /syslog/i );
-	$aOpt{'loglevel'} ||= 'warning';
+    # Force logging to file if this is windoze and user requested syslog mechanism
+    $aOpt{'log'} = 'file' if( $aOpt{'ostype'} eq 'windoze' && $aOpt{'log'} =~ /syslog/i );
+    $aOpt{'loglevel'} ||= 'warning';
 
-	if( ! ref $aOpt{'log'} ) {
-		my($method, @options) = split ',', delete $aOpt{'log'};
-		my $logclass = 'Device/Modem/Log/'.ucfirst(lc $method).'.pm';
-		my $package = 'Device::Modem::Log::'.ucfirst lc $method;
-		eval { require $logclass; };
-		unless($@) {
-			$aOpt{'_log'} = $package->new( $class, @options );
-		}
-	} else {
-		# User passed an already instanced log object
-		$aOpt{'_log'} = $aOpt{'log'};
-	}
+    if( ! ref $aOpt{'log'} ) {
+        my($method, @options) = split ',', delete $aOpt{'log'};
+        my $logclass = 'Device/Modem/Log/'.ucfirst(lc $method).'.pm';
+        my $package = 'Device::Modem::Log::'.ucfirst lc $method;
+        eval { require $logclass; };
+        unless($@) {
+            $aOpt{'_log'} = $package->new( $class, @options );
+        }
+    } else {
 
-	if( ref $aOpt{'_log'} && $aOpt{'_log'}->can('loglevel') ) {
-		$aOpt{'_log'}->loglevel($aOpt{'loglevel'});
-	}
+        # User passed an already instanced log object
+        $aOpt{'_log'} = $aOpt{'log'};
+    }
 
-	bless \%aOpt, $class;                   # Instance $class object
+    if( ref $aOpt{'_log'} && $aOpt{'_log'}->can('loglevel') ) {
+        $aOpt{'_log'}->loglevel($aOpt{'loglevel'});
+    }
+
+    bless \%aOpt, $class;                   # Instance $class object
 }
 
 sub attention {
-	my $self = shift;
-	$self->log->write('info', 'sending attention sequence...');
+    my $self = shift;
+    $self->log->write('info', 'sending attention sequence...');
 
-	# Send attention sequence
-	$self->atsend('+++');
+    # Send attention sequence
+    $self->atsend('+++');
 
-	# Wait 200 milliseconds
-	$self->wait(200);
-	$self->answer();
+    # Wait 200 milliseconds
+    $self->wait(200);
+    $self->answer();
 }
 
 #
@@ -132,545 +133,543 @@ sub attention {
 # [ see store_number() ]
 #
 sub dial {
-	my($self, $number, $timeout) = @_;
-	my $ok = 0;
+    my($self, $number, $timeout) = @_;
+    my $ok = 0;
 
-	# Default timeout in seconds
-	$timeout ||= 30;
+    # Default timeout in seconds
+    $timeout ||= 30;
 
-	# Check if we have already dialed some number...
-	if( $self->flag('CARRIER') ) {
-		$self->log->write( 'warning', 'line is already connected, ignoring dial()' );
-		return;
-	}
+    # Check if we have already dialed some number...
+    if( $self->flag('CARRIER') ) {
+        $self->log->write( 'warning', 'line is already connected, ignoring dial()' );
+        return;
+    }
 
-	# Check if no number supplied
-	if( ! defined $number ) {
-		#
-		# XXX Here we could enable ATDL command (dial last number)
-		#
-		$self->log->write( 'warning', 'cannot dial without a number!' );
-		return;
-	}
+    # Check if no number supplied
+    if( ! defined $number ) {
 
-	# Remove all non number chars plus some others allowed
-	$number =~ s/[^0-9,\(\)\*\-\s]//g;
+        #
+        # XXX Here we could enable ATDL command (dial last number)
+        #
+        $self->log->write( 'warning', 'cannot dial without a number!' );
+        return;
+    }
 
-	# Dial number and wait for response
-	if( length $number == 1 ) {
-		$self->log->write('info', 'dialing address book number ['.$number.']' );
-		$self->atsend( 'ATDS' . $number . CR );
-	} else {
-		$self->log->write('info', 'dialing number ['.$number.']' );
-		$self->atsend( 'ATDT' . $number . CR );
-	}
+    # Remove all non number chars plus some others allowed
+    $number =~ s/[^0-9,\(\)\*\-\s]//g;
 
-	# XXX Check response times here (timeout!)
-	my $ans = $self->answer(undef, $timeout * 1000 );
+    # Dial number and wait for response
+    if( length $number == 1 ) {
+        $self->log->write('info', 'dialing address book number ['.$number.']' );
+        $self->atsend( 'ATDS' . $number . CR );
+    } else {
+        $self->log->write('info', 'dialing number ['.$number.']' );
+        $self->atsend( 'ATDT' . $number . CR );
+    }
 
-	$ok = 1 if index( $ans, 'CONNECT' ) > -1;
+    # XXX Check response times here (timeout!)
+    my $ans = $self->answer(undef, $timeout * 1000 );
 
-	# Turn on/off `CARRIER' flag
-	$self->flag('CARRIER', $ok);
+    if( (index($ans,'CONNECT') > -1) || (index($ans,'RING') > -1) ) {
+        $ok = 1;
+    }
 
-	$self->log->write('info', 'dialing result = '.$ok);
-	return wantarray ? ($ok, $ans) : $ok;
+    # Turn on/off `CARRIER' flag
+    $self->flag('CARRIER', $ok);
+
+    $self->log->write('info', 'dialing result = '.$ok);
+    return wantarray ? ($ok, $ans) : $ok;
 }
 
 # Enable/disable local echo of commands (enabling echo can cause everything else to fail, I think)
 sub echo {
-	my($self, $lEnable) = @_;
+    my($self, $lEnable) = @_;
 
-	$self->log->write( 'info', ( $lEnable ? 'enabling' : 'disabling' ) . ' echo' );
-	$self->atsend( ($lEnable ? 'ATE1' : 'ATE0') . CR );
+    $self->log->write( 'info', ( $lEnable ? 'enabling' : 'disabling' ) . ' echo' );
+    $self->atsend( ($lEnable ? 'ATE1' : 'ATE0') . CR );
 
-	$self->answer('OK');
+    $self->answer('OK');
 }
 
 # Terminate current call (XXX not tested)
 sub hangup {
-	my $self = shift;
+    my $self = shift;
 
-	$self->log->write('info', 'hanging up...');
-	$self->attention();
-	$self->atsend( 'ATH0' . CR );
-	$self->_reset_flags();
-	$self->answer();
+    $self->log->write('info', 'hanging up...');
+    $self->attention();
+    $self->atsend( 'ATH0' . CR );
+    $self->_reset_flags();
+    $self->answer();
 }
 
 # Checks if modem is enabled (for now, it works ok for modem OFF/ON case)
 sub is_active {
-	my $self = shift;
-	my $lOk;
+    my $self = shift;
+    my $lOk;
 
-	$self->log->write('info', 'testing modem activity on port '.$self->options->{'port'} );
+    $self->log->write('info', 'testing modem activity on port '.$self->options->{'port'} );
 
-	# Modem is active if already connected to a line
-	if( $self->flag('CARRIER') ) {
+    # Modem is active if already connected to a line
+    if( $self->flag('CARRIER') ) {
 
-		$self->log->write('info', 'carrier is '.$self->flag('CARRIER').', modem is connected, it should be active');
-		$lOk = 1;
+        $self->log->write('info', 'carrier is '.$self->flag('CARRIER').', modem is connected, it should be active');
+        $lOk = 1;
 
-	} else {
+    } else {
 
-		# XXX Old mode to test modem ... 
-		# Try sending an echo enable|disable command
-		#$self->attention();
-		#$self->verbose(0);
-		#$lOk = $self->verbose(1);
+        # XXX Old mode to test modem ...
+        # Try sending an echo enable|disable command
+        #$self->attention();
+        #$self->verbose(0);
+        #$lOk = $self->verbose(1);
 
-		# If DSR signal is on, modem is active
-		my %sig = $self->status();
-		$lOk = $sig{DSR};
-		undef %sig;
+        # If DSR signal is on, modem is active
+        my %sig = $self->status();
+        $lOk = $sig{DSR};
+        undef %sig;
 
-		# If we have no success, try to reset
-		if( ! $lOk ) {
-			$self->log->write('warning', 'modem not responding... trying to reset');
-			$lOk = $self->reset();
-		}
+        # If we have no success, try to reset
+        if( ! $lOk ) {
+            $self->log->write('warning', 'modem not responding... trying to reset');
+            $lOk = $self->reset();
+        }
 
-	}
+    }
 
-	$self->log->write('info', 'modem reset result = '.$lOk);
+    $self->log->write('info', 'modem reset result = '.$lOk);
 
-	return $lOk;
+    return $lOk;
 }
 
 # Take modem off hook, prepare to dial
 sub offhook {
-	my $self = shift;
+    my $self = shift;
 
-	$self->log->write('info', 'taking off hook');
-	$self->atsend( 'ATH1' . CR );
+    $self->log->write('info', 'taking off hook');
+    $self->atsend( 'ATH1' . CR );
 
-	$self->flag('OFFHOOK', 1);
+    $self->flag('OFFHOOK', 1);
 
-	return 1;
+    return 1;
 }
 
 # Get/Set S* registers value:  S_register( number [, new_value] )
 # returns undef on failure ( zero is a good value )
 sub S_register {
-	my $self = shift;
-	my $register = shift;
-	my $value = 0;
+    my $self = shift;
+    my $register = shift;
+    my $value = 0;
 
-	return unless $register;
+    return unless $register;
 
-	my $ok;
+    my $ok;
 
-	# If `new_value' supplied, we want to update value of this register
-	if( @_ ) {
+    # If `new_value' supplied, we want to update value of this register
+    if( @_ ) {
 
-		my $new_value = shift;
-		$new_value =~ s|\D||g;
-		$self->log->write('info', 'storing value ['.$new_value.'] into register S'.$register);
-		$self->atsend( sprintf( 'AT S%02d=%d' . CR, $register, $new_value ) );
+        my $new_value = shift;
+        $new_value =~ s|\D||g;
+        $self->log->write('info', 'storing value ['.$new_value.'] into register S'.$register);
+        $self->atsend( sprintf( 'AT S%02d=%d' . CR, $register, $new_value ) );
 
+        $value = ( index( $self->answer, 'OK' ) != -1 ) ? $new_value : undef;
 
-		$value = ( index( $self->answer, 'OK' ) != -1 ) ? $new_value : undef;
+    } else {
 
-	} else {
+        $self->atsend( sprintf( 'AT S%d?' . CR, $register ) );
+        ($ok, $value) = $self->parse_answer();
 
-		$self->atsend( sprintf( 'AT S%d?' . CR, $register ) );
-		($ok, $value) = $self->parse_answer();
+        if( index($ok, 'OK') != -1 ) {
+            $self->log->write('info', 'value of S'.$register.' register seems to be ['.$value.']');
+        } else {
+            $value = undef;
+            $self->log->write('error', 'error reading value of S'.$register.' register');
+        }
 
-		if( index($ok, 'OK') != -1 ) {
-			$self->log->write('info', 'value of S'.$register.' register seems to be ['.$value.']');
-		} else {
-			$value = undef;
-			$self->log->write('error', 'error reading value of S'.$register.' register');
-		}
+    }
 
-	}
+    # Return updated value of register
+    $self->log->write('info', 'S'.$register.' = '.$value);
 
-	# Return updated value of register
-	$self->log->write('info', 'S'.$register.' = '.$value);
-
-	return $value;
+    return $value;
 }
 
 # Repeat the last commands (this comes gratis with `A/' at-command)
 sub repeat {
-	my $self = shift;
+    my $self = shift;
 
-	$self->log->write('info', 'repeating last command' );
-	$self->atsend( 'A/' . CR );
+    $self->log->write('info', 'repeating last command' );
+    $self->atsend( 'A/' . CR );
 
-	$self->answer();
+    $self->answer();
 }
 
 # Complete modem reset
 sub reset {
-	my $self = shift;
+    my $self = shift;
 
-	$self->log->write('warning', 'resetting modem on '.$self->{'port'} );
-	$self->hangup();
-	$self->send_init_string();
-	$self->_reset_flags();
-	return $self->answer();
+    $self->log->write('warning', 'resetting modem on '.$self->{'port'} );
+    $self->hangup();
+    $self->send_init_string();
+    $self->_reset_flags();
+    return $self->answer();
 }
 
 # Return an hash with the status of main modem signals
 sub status {
-	my $self = shift;
-	$self->log->write('info', 'getting modem line status on '.$self->{'port'});
+    my $self = shift;
+    $self->log->write('info', 'getting modem line status on '.$self->{'port'});
 
-	# This also relies on Device::SerialPort
-	my $status = $self->port->modemlines();
+    # This also relies on Device::SerialPort
+    my $status = $self->port->modemlines();
 
-	# See top of module for these constants, exported by (Win32|Device)::SerialPort
-	my %signal = (
-		CTS  => $status & Device::Modem::MS_CTS_ON,
-		DSR  => $status & Device::Modem::MS_DSR_ON,
-		RING => $status & Device::Modem::MS_RING_ON,
-		RLSD => $status & Device::Modem::MS_RLSD_ON
-	);
+ # See top of module for these constants, exported by (Win32|Device)::SerialPort
+    my %signal = (
+        CTS  => $status & Device::Modem::MS_CTS_ON,
+        DSR  => $status & Device::Modem::MS_DSR_ON,
+        RING => $status & Device::Modem::MS_RING_ON,
+        RLSD => $status & Device::Modem::MS_RLSD_ON
+      );
 
-	$self->log->write('info', 'modem on '.$self->{'port'}.' status is ['.$status.']');
-	$self->log->write('info', "CTS=$signal{CTS} DSR=$signal{DSR} RING=$signal{RING} RLSD=$signal{RLSD}");
+    $self->log->write('info', 'modem on '.$self->{'port'}.' status is ['.$status.']');
+    $self->log->write('info', "CTS=$signal{CTS} DSR=$signal{DSR} RING=$signal{RING} RLSD=$signal{RLSD}");
 
-	return %signal;
+    return %signal;
 }
 
 # Of little use here, but nice to have it
 # restore_factory_settings( profile )
 # profile can be 0 or 1
 sub restore_factory_settings {
-	my $self = shift;
-	my $profile = shift;
-	$profile = 0 unless defined $profile;
+    my $self = shift;
+    my $profile = shift;
+    $profile = 0 unless defined $profile;
 
-	$self->log->write('warning', 'restoring factory settings '.$profile.' on '.$self->{'port'} );
-	$self->atsend( 'AT&F'.$profile . CR);
+    $self->log->write('warning', 'restoring factory settings '.$profile.' on '.$self->{'port'} );
+    $self->atsend( 'AT&F'.$profile . CR);
 
-	$self->answer();
+    $self->answer();
 }
 
 # Store telephone number in modem's internal address book, to dial later
 # store_number( position, number )
 sub store_number {
-	my( $self, $position, $number ) = @_;
-	my $ok = 0;
+    my( $self, $position, $number ) = @_;
+    my $ok = 0;
 
-	# Check parameters
-	unless( defined($position) && $number ) {
-		$self->log->write('warning', 'store_number() called with wrong parameters');
-		return $ok;
-	}
+    # Check parameters
+    unless( defined($position) && $number ) {
+        $self->log->write('warning', 'store_number() called with wrong parameters');
+        return $ok;
+    }
 
-	$self->log->write('info', 'storing number ['.$number.'] into memory ['.$position.']');
+    $self->log->write('info', 'storing number ['.$number.'] into memory ['.$position.']');
 
-	# Remove all non-numerical chars from position and number
-	$position =~ s/\D//g;
-	$number   =~ s/[^0-9,]//g;
+    # Remove all non-numerical chars from position and number
+    $position =~ s/\D//g;
+    $number   =~ s/[^0-9,]//g;
 
-	$self->atsend( sprintf( 'AT &Z%d=%s' . CR, $position, $number ) );
+    $self->atsend( sprintf( 'AT &Z%d=%s' . CR, $position, $number ) );
 
-	if( index( $self->answer(), 'OK' ) != -1 ) {
-		$self->log->write('info', 'stored number ['.$number.'] into memory ['.$position.']');
-		$ok = 1;
-	} else {
-		$self->log->write('warning', 'error storing number ['.$number.'] into memory ['.$position.']');
-		$ok = 0;
-	}
+    if( index( $self->answer(), 'OK' ) != -1 ) {
+        $self->log->write('info', 'stored number ['.$number.'] into memory ['.$position.']');
+        $ok = 1;
+    } else {
+        $self->log->write('warning', 'error storing number ['.$number.'] into memory ['.$position.']');
+        $ok = 0;
+    }
 
-	return $ok;
+    return $ok;
 }
 
 # Enable/disable verbose response messages against numerical response messages
 # XXX I need to manage also numerical values...
 sub verbose {
-	my($self, $lEnable) = @_;
+    my($self, $lEnable) = @_;
 
-	$self->log->write( 'info', ( $lEnable ? 'enabling' : 'disabling' ) . ' verbose messages' );
-	$self->atsend( ($lEnable ? 'ATQ0V1' : 'ATQ0V0') . CR );
+    $self->log->write( 'info', ( $lEnable ? 'enabling' : 'disabling' ) . ' verbose messages' );
+    $self->atsend( ($lEnable ? 'ATQ0V1' : 'ATQ0V0') . CR );
 
-	$self->answer('OK');
+    $self->answer('OK');
 }
 
-
 sub wait {
-	my( $self, $msec ) = @_;
+    my( $self, $msec ) = @_;
 
-	$self->log->write('debug', 'waiting for '.$msec.' msecs');
+    $self->log->write('debug', 'waiting for '.$msec.' msecs');
 
-	# Perhaps Time::HiRes here is not so useful, since I tested `select()' system call also on Windows
-	select( undef, undef, undef, $msec / 1000 );
-	return 1;
+# Perhaps Time::HiRes here is not so useful, since I tested `select()' system call also on Windows
+    select( undef, undef, undef, $msec / 1000 );
+    return 1;
 
 }
 
 # Set a named flag. Flags are now: OFFHOOK, CARRIER
 sub flag {
-	my $self = shift;
-	my $cFlag = uc shift;
+    my $self = shift;
+    my $cFlag = uc shift;
 
-	$self->{'_flags'}->{$cFlag} = shift() if @_;
+    $self->{'_flags'}->{$cFlag} = shift() if @_;
 
-	$self->{'_flags'}->{$cFlag};
+    $self->{'_flags'}->{$cFlag};
 }
 
 # reset internal flags that tell the status of modem (XXX to be extended)
 sub _reset_flags {
-	my $self = shift();
+    my $self = shift();
 
-	map { $self->flag($_, 0) }
-		'OFFHOOK', 'CARRIER';
+    map { $self->flag($_, 0) }
+      'OFFHOOK', 'CARRIER';
 }
 
 # initialize modem with some basic commands (XXX &C0)
 # send_init_string( [my_init_string] )
 # my_init_string goes without 'AT' prefix
 sub send_init_string {
-	my($self, $cInit) = @_;
-	$self->attention();
-	$cInit = $self->options->{'init_string'} unless defined $cInit;
-	$self->atsend('AT '.$cInit. CR );
-	$self->answer();
+    my($self, $cInit) = @_;
+    $self->attention();
+    $cInit = $self->options->{'init_string'} unless defined $cInit;
+    $self->atsend('AT '.$cInit. CR );
+    $self->answer();
 }
 
 # returns log object reference or nothing if it is not defined
 sub log {
-	my $me = shift;
-	if( ref $me->{'_log'} ) {
-		return $me->{'_log'};
-	} else {
-		return {};
-	}
+    my $me = shift;
+    if( ref $me->{'_log'} ) {
+        return $me->{'_log'};
+    } else {
+        return {};
+    }
 }
 
 # instances (Device|Win32)::SerialPort object and initializes communications
 sub connect {
-	my $me = shift();
+    my $me = shift();
 
-	my %aOpt = ();
-	if( @_ ) {
-		%aOpt = @_;
-	}
+    my %aOpt = ();
+    if( @_ ) {
+        %aOpt = @_;
+    }
 
-	my $lOk = 0;
+    my $lOk = 0;
 
-	# Set default values if missing
-	$aOpt{'baudrate'} ||= $Device::Modem::BAUDRATE;
-	$aOpt{'databits'} ||= $Device::Modem::DATABITS;
-	$aOpt{'parity'}   ||= $Device::Modem::PARITY;
-	$aOpt{'stopbits'} ||= $Device::Modem::STOPBITS;
+    # Set default values if missing
+    $aOpt{'baudrate'} ||= $Device::Modem::BAUDRATE;
+    $aOpt{'databits'} ||= $Device::Modem::DATABITS;
+    $aOpt{'parity'}   ||= $Device::Modem::PARITY;
+    $aOpt{'stopbits'} ||= $Device::Modem::STOPBITS;
 
-	# Store communication options in object
-	$me->{'_comm_options'} = \%aOpt;
+    # Store communication options in object
+    $me->{'_comm_options'} = \%aOpt;
 
-	# Connect on serial (use different mod for win32)
-	if( $me->ostype eq 'windoze' ) {
-		$me->port( new Win32::SerialPort($me->{'port'}) );
-	} else {
-		$me->port( new Device::SerialPort($me->{'port'}) );
-	}
+    # Connect on serial (use different mod for win32)
+    if( $me->ostype eq 'windoze' ) {
+        $me->port( new Win32::SerialPort($me->{'port'}) );
+    } else {
+        $me->port( new Device::SerialPort($me->{'port'}) );
+    }
 
-	# Check connection
-	unless( ref $me->port ) {
-		$me->log->write( 'error', '*FAILED* connect on '.$me->{'port'} );
-		return $lOk;
-	}
+    # Check connection
+    unless( ref $me->port ) {
+        $me->log->write( 'error', '*FAILED* connect on '.$me->{'port'} );
+        return $lOk;
+    }
 
-	# Set communication options
-	my $oPort = $me->port;
-	$oPort -> baudrate ( $me->options->{'baudrate'} );
-	$oPort -> databits ( $me->options->{'databits'} );
-	$oPort -> stopbits ( $me->options->{'stopbits'} );
-	$oPort -> parity   ( $me->options->{'parity'}   );
+    # Set communication options
+    my $oPort = $me->port;
+    $oPort -> baudrate ( $me->options->{'baudrate'} );
+    $oPort -> databits ( $me->options->{'databits'} );
+    $oPort -> stopbits ( $me->options->{'stopbits'} );
+    $oPort -> parity   ( $me->options->{'parity'}   );
 
-	# Non configurable options
-	$oPort -> buffers         ( 10000, 10000 );
-	$oPort -> handshake       ( 'none' );
-	$oPort -> read_const_time ( 100 );           # was 500
-	$oPort -> read_char_time  ( 10 );
+    # Non configurable options
+    $oPort -> buffers         ( 10000, 10000 );
+    $oPort -> handshake       ( 'none' );
+    $oPort -> read_const_time ( 100 );           # was 500
+    $oPort -> read_char_time  ( 10 );
 
-	$oPort -> are_match       ( 'OK' );
-	$oPort -> lookclear;
+    $oPort -> are_match       ( 'OK' );
+    $oPort -> lookclear;
 
-	$oPort -> write_settings;
-	$oPort -> purge_all;
+    $oPort -> write_settings;
+    $oPort -> purge_all;
 
-	$me-> log -> write('info', 'sending init string...' );
+    $me-> log -> write('info', 'sending init string...' );
 
-	# Set default initialization string if none supplied
-	$me->options->{'init_string'} ||= 'H0 Z S7=45 S0=0 Q0 V1 E0 &C0 X4';
+    # Set default initialization string if none supplied
+    $me->options->{'init_string'} ||= 'H0 Z S7=45 S0=0 Q0 V1 E0 &C0 X4';
 
-	$me-> send_init_string( $me->options->{'init_string'} );
-	$me-> _reset_flags();
+    $me-> send_init_string( $me->options->{'init_string'} );
+    $me-> _reset_flags();
 
-	# Disable local echo
-	$me-> echo(0);
+    # Disable local echo
+    $me-> echo(0);
 
-	$me-> log -> write('info', 'Ok connected' );
-	$me-> {'CONNECTED'} = 1;
+    $me-> log -> write('info', 'Ok connected' );
+    $me-> {'CONNECTED'} = 1;
 
 }
 
 # $^O is stored into object
 sub ostype {
-	my $self = shift;
-	$self->{'ostype'};
+    my $self = shift;
+    $self->{'ostype'};
 }
 
 # returns Device::SerialPort reference to hash options
 sub options {
-	my $self = shift();
-	@_ ? $self->{'_comm_options'} = shift()
-	   : $self->{'_comm_options'};
+    my $self = shift();
+    @_ ? $self->{'_comm_options'} = shift()
+      : $self->{'_comm_options'};
 }
 
 # returns Device::SerialPort object handle
 sub port {
-	my $self = shift();
-	@_ ? $self->{'_comm_object'} = shift()
-	   : $self->{'_comm_object'};
+    my $self = shift();
+    @_ ? $self->{'_comm_object'} = shift()
+      : $self->{'_comm_object'};
 }
 
 # disconnect serial port
 sub disconnect {
-	my $me = shift;
-	$me->port->close();
-	$me->log->write('info', 'Disconnected from '.$me->{'port'} );
+    my $me = shift;
+    $me->port->close();
+    $me->log->write('info', 'Disconnected from '.$me->{'port'} );
 }
 
 # Send AT command to device on serial port (command must include CR for now)
 sub atsend {
-	my( $me, $msg ) = @_;
-	my $cnt = 0;
+    my( $me, $msg ) = @_;
+    my $cnt = 0;
 
-	# Write message on port
-	$me->port->purge_all();
-	$cnt = $me->port->write($msg);
-	$me->wait(400);
+    # Write message on port
+    $me->port->purge_all();
+    $cnt = $me->port->write($msg);
+    $me->wait(400);
 
-	$me->port->write_drain() unless $me->ostype eq 'windoze';
-	$me->log->write('debug', 'atsend: wrote '.$cnt.'/'.length($msg).' chars');
+    $me->port->write_drain() unless $me->ostype eq 'windoze';
+    $me->log->write('debug', 'atsend: wrote '.$cnt.'/'.length($msg).' chars');
 
-	# If wrote all chars of `msg', we are successful
-	return $cnt == length $msg;
+    # If wrote all chars of `msg', we are successful
+    return $cnt == length $msg;
 }
 
 # answer() takes strings from the device until a pattern
 # is encountered or a timeout happens.
 sub _answer {
-	my $me = shift;
-	my($expect, $timeout) = @_;
-	my $time_slice = 50;                       # single cycle wait time
+    my $me = shift;
+    my($expect, $timeout) = @_;
+    my $time_slice = 50;                       # single cycle wait time
 
-	# If we expect something, we must first match against serial input
-	my $done = (defined $expect and $expect ne '');
+    # If we expect something, we must first match against serial input
+    my $done = (defined $expect and $expect ne '');
 
-	#$me->log->write('debug', 'answer: expecting ['.($expect||'').']'.($timeout ? ' or '.($timeout/1000).' seconds timeout' : '' ) );
+#$me->log->write('debug', 'answer: expecting ['.($expect||'').']'.($timeout ? ' or '.($timeout/1000).' seconds timeout' : '' ) );
 
-	# Main read cycle
-	my $cycles = 0;
-	my $idle_cycles = 0;
-	my $answer;
-	my $start_time = time();
-	my $end_time   = 0;
+    # Main read cycle
+    my $cycles = 0;
+    my $idle_cycles = 0;
+    my $answer;
+    my $start_time = time();
+    my $end_time   = 0;
 
-	# If timeout was defined, check max time (timeout is in milliseconds)
-	#$me->log->write('debug', 'answer: timeout value is '.$timeout);
+    # If timeout was defined, check max time (timeout is in milliseconds)
+    #$me->log->write('debug', 'answer: timeout value is '.$timeout);
 
-	if( defined $timeout && $timeout > 0 ) {
-		$end_time = $start_time + ($timeout / 1000);
-		#$me->log->write( debug => 'answer: end time set to '.$end_time );
-	}
+    if( defined $timeout && $timeout > 0 ) {
+        $end_time = $start_time + ($timeout / 1000);
 
-	do {
+        #$me->log->write( debug => 'answer: end time set to '.$end_time );
+    }
 
-		my($howmany, $what) = $me->port->read(10);
+    do {
 
-		# Timeout count incremented only on empty readings
-		if( defined $what && $howmany > 0 ) {
+        my($howmany, $what) = $me->port->read(10);
 
-			# Add received chars to answer string
-			$answer .= $what;
+        # Timeout count incremented only on empty readings
+        if( defined $what && $howmany > 0 ) {
 
-			# Check if buffer matches "expect string"
-			if( defined $expect ) {
-				$done = ( defined $answer && $answer =~ $expect ) ? 1 : 0;
-			}
+            # Add received chars to answer string
+            $answer .= $what;
 
-			#$me->log->write('debug', 'time_slice='.$time_slice);
-			$me->wait($time_slice) unless $done;
+            # Check if buffer matches "expect string"
+            if( defined $expect ) {
+                $done = ( defined $answer && $answer =~ $expect ) ? 1 : 0;
+            }
 
-		} else {
+            #$me->log->write('debug', 'time_slice='.$time_slice);
+            $me->wait($time_slice) unless $done;
 
-			$done = 1;
+        # Check if we reached max time for timeout (only if end_time is defined)
+        } elsif( $end_time > 0 ) {
 
-		}
+            $done = time() >= $end_time ? 1 : 0;
 
-		# Check if we reached max time for timeout (only if end_time is defined)
-		if( $howmany == 0 && $end_time > 0 ) {
-			$done = ( time() >= $end_time ) ? 1 : 0;
-		}
+        # Else we have done
+        } else {
 
-		#$me->log->write('debug', 'done = '.$done );
-		#$me->log->write('debug', 'end_time='.$end_time.' now='.time().' start_time='.$start_time );
+            $done = 1;
+        }
 
-	} while (not $done);
+#$me->log->write('debug', 'done = '.$done );
+#$me->log->write('debug', 'end_time='.$end_time.' now='.time().' start_time='.$start_time );
 
-	$me->log->write('debug', 'answer: read ['.($answer||'').']' );
+      } while (not $done);
 
-	# Flush receive and trasmit buffers
-	$me->port->purge_all;
+    $me->log->write('debug', 'answer: read ['.($answer||'').']' );
 
-	return $answer;
+    # Flush receive and trasmit buffers
+    $me->port->purge_all;
 
-} 
+    return $answer;
+
+}
 
 sub answer {
 
-	my $me = shift();
-	my $answer = $me->_answer(@_);
+    my $me = shift();
+    my $answer = $me->_answer(@_);
 
-	# Trim result of beginning and ending CR+LF (XXX)
-	if( defined $answer ) {
-		$answer =~ s/^[\r\n]+//;
-		$answer =~ s/[\r\n]+$//;
-	}
+    # Trim result of beginning and ending CR+LF (XXX)
+    if( defined $answer ) {
+        $answer =~ s/^[\r\n]+//;
+        $answer =~ s/[\r\n]+$//;
+    }
 
-	$me->log->write('info', 'answer: `'.($answer||'').'\'' );
+    $me->log->write('info', 'answer: `'.($answer||'').'\'' );
 
-	return $answer;
+    return $answer;
 }
 
-
 # parse_answer() cleans out answer() result as response code +
 # useful information (useful in informative commands, for example
 # Gsm command AT+CGMI)
 sub parse_answer {
-	my $me = shift;
+    my $me = shift;
 
-	my $buff = $me->answer( @_ );
+    my $buff = $me->answer( @_ );
 
-	# Separate response code from information
-	my @buff = split /[\r\n]+/o, $buff;
+    # Separate response code from information
+    my @buff = split /[\r\n]+/o, $buff;
 
-	# Remove all empty lines before/after response
-	shift @buff while $buff[0]  =~ /^[\r\n]+/o;
-	pop   @buff while $buff[-1] =~ /^[\r\n]+/o;
+    # Remove all empty lines before/after response
+    shift @buff while $buff[0]  =~ /^[\r\n]+/o;
+    pop   @buff while $buff[-1] =~ /^[\r\n]+/o;
 
-	# Extract responde code
-	my $code = pop @buff;
-	$buff = join( CR, @buff );
+    # Extract responde code
+    my $code = pop @buff;
+    $buff = join( CR, @buff );
 
-	return
-		wantarray
-		? ($code, @buff)
-		: $buff;
+    return
+      wantarray
+      ? ($code, @buff)
+      : $buff;
 
 }
 
-
-
 1;
 
-
 =head1 NAME
 
 Device::Modem - Perl extension to talk to modem devices connected via serial port
@@ -1344,3 +1343,4 @@ perl
 
 =cut
 
+# vim: set ts=4 sw=4 tw=120 nowrap nu
